<!DOCTYPE html>
<html lang="en">
<head>
	{% block head %}
	<meta charset="utf-8" />
	<title>{% block title %}{% endblock %}</title>
	<link rel="stylesheet" href="{{ url_for('.static', filename='960.css') }}"/>
	<link rel="stylesheet" href="{{ url_for('.static', filename='main.css') }}"/>
	<script src="http://ajax.googleapis.com/ajax/libs/jquery/1.8.2/jquery.min.js" type="text/javascript"></script>
	<script src="http://ajax.googleapis.com/ajax/libs/jqueryui/1.9.2/jquery-ui.min.js" type="text/javascript"></script>
	<script src="{{ url_for('.static', filename='flashes.js') }}" type="text/javascript"></script>
	{% endblock %}
</head>
<body>
	<header>
		{# These are in a <header> tag because it's not relevant to the page. #}
		{# It's nice to easily have flashed messages in every page. #}
		{% with messages = get_flashed_messages(with_categories=true) %}
		<div id="flashes">
		{% if messages %}	
			{% for category, message in messages %}
			<p class="flash flash-{{ category }} flash-num-{{ loop.index }}">{{ message }}</p>
			{% endfor %}
		{% endif %}
		</div>
		{% endwith %}
	</header>
	<content>
	<div class="container_12 content">
		<div class="push_3 grid_6" id="stats">
			<br />
			{% block content %}{% endblock %}
		</div>
	</div>
	</content>
	<footer>
	<div id="credits">
		An <a href="https://github.com/plausibility">@plausibility</a> &amp; <a href="https://github.com/huey-">@huey-</a> production.
	</div>
	</footer>
</body>
</html>
