""" The chr api is a neat interface to a neat program.

    Lean, mean, and straight to the point, it aims to get work **done**.

    In and out in 60 seconds is the idea;
    we keep it simple and easy to work with.
"""